@@ -3,7 +3,7 @@ import asyncore
 from pyftpdlib import ftpserver
 from ftpcloudfs.utils import smart_str
 from server import RackspaceCloudAuthorizer
-import logging
+from multiprocessing.managers import RemoteError
 
 class MyDTPHandler(ftpserver.DTPHandler):
     def send(self, data):
@@ -39,7 +39,8 @@ class MyFTPHandler(ftpserver.FTPHandler):
     def process_command(self, cmd, *args, **kwargs):
         '''Flush the FS cache with every new FTP command'''
         if self.fs:
-            self.fs.flush()
+            if not self.fs.single_cache:
+                self.fs.flush()
             self.fs.connection.real_ip = self.remote_ip
         super(MyFTPHandler, self).process_command(cmd, *args, **kwargs)
 
@@ -57,17 +58,21 @@ class MyFTPHandler(ftpserver.FTPHandler):
     def handle(self):
         """Track the ip and check max cons per ip (if needed)"""
         if self.server.max_cons_per_ip and self.remote_ip and self.shared_ip_map != None:
-            self.shared_lock.acquire()
-            count = self.shared_ip_map.get(self.remote_ip, 0)
-            self.shared_ip_map[self.remote_ip] = count + 1
-            self.shared_lock.release()
+            try:
+                self.shared_lock.acquire()
+                count = self.shared_ip_map.get(self.remote_ip, 0)
+                self.shared_ip_map[self.remote_ip] = count + 1
+                self.shared_lock.release()
+            except RemoteError, e:
+                self.log("connection tracking failed: %s" % e)
 
-            logging.debug("connection track: %s -> %s" % (self.remote_ip, count+1))
+            self.logline("connection track: %s -> %s" % (self.remote_ip, count+1))
 
             if self.shared_ip_map[self.remote_ip] > self.server.max_cons_per_ip:
                 self.handle_max_cons_per_ip()
                 return
-            logging.debug("connected, shared ip map: %s" % self.shared_ip_map)
+
+            self.logline("connected, shared ip map: %s" % self.shared_ip_map)
 
         super(MyFTPHandler, self).handle()
 
@@ -75,13 +80,16 @@ class MyFTPHandler(ftpserver.FTPHandler):
         """Remove the ip from the shared map before calling close"""
         if not self._closed and self.server.max_cons_per_ip and self.shared_ip_map != None:
             if self.remote_ip in self.shared_ip_map:
-                self.shared_lock.acquire()
-                self.shared_ip_map[self.remote_ip] -= 1
-                if self.shared_ip_map[self.remote_ip] <= 0:
-                    del self.shared_ip_map[self.remote_ip]
-                self.shared_lock.release()
+                try:
+                    self.shared_lock.acquire()
+                    self.shared_ip_map[self.remote_ip] -= 1
+                    if self.shared_ip_map[self.remote_ip] <= 0:
+                        del self.shared_ip_map[self.remote_ip]
+                    self.shared_lock.release()
+                except RemoteError, e:
+                    self.log("connection tracking cleanup failed: %s" % e)
 
-                logging.debug("disconnected, shared ip map: %s" % self.shared_ip_map)
+                self.logline("disconnected, shared ip map: %s" % self.shared_ip_map)
 
         super(MyFTPHandler, self).close()
 
